
/*
 *   Driver program created: Thu Feb  5 14:18:38 EST 2004
 *   Richard B. Johnson          rjohnson@analogic.com
 *   This program is not guaranteed to do anything useful.
 *   In fact, it can be used for harmful things, including
 *   disguising a machine.
 *
 *   Use at your own risk. Have fun!
 */

#ifndef __KERNEL__
#define __KERNEL__
#endif

#ifndef MODULE
#define MODULE
#endif

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/utsname.h>
#include <asm/uaccess.h>
#include "unamedrvr.h"


#define SFO struct file_operations
static SFO *ops;
extern struct new_utsname system_utsname;
int __init init_module(void);
void cleanup_module(void);
static int dev_close(struct inode *, struct file *);
static int dev_ioctl(struct inode *, struct file *, unsigned int, unsigned long);
static int dev_open(struct inode *, struct file *);
static const char dev[]="UNAME";

MODULE_AUTHOR("Richard B. Johnson");
MODULE_LICENSE("Dual BSD/GPL");
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *   This handles device-specifc functions.
 */
#define GETPARS(member) \
   cp  = system_utsname.member; \
   len = sizeof(system_utsname.member)

static int dev_ioctl(struct inode *inp, struct file *fp, unsigned int cmd, unsigned long arg)
{
    char *cp;
    size_t len;
    switch(cmd)
    {
    case SET_UNAME_SYSNAME:
        GETPARS(sysname);
        break;
    case SET_UNAME_NODENAME:
        GETPARS(nodename);
        break;
    case SET_UNAME_RELEASE:
        GETPARS(release);
        break;
    case SET_UNAME_VERSION: 
        GETPARS(version);
        break;
    case SET_UNAME_MACHINE:
        GETPARS(machine); 
        break;
    case SET_UNAME_DOMAIN:
        GETPARS(domainname);
        break;
    default:
        printk(KERN_ERR"%s : Invalid parameter\n", dev);
        return -EINVAL;
    }
    {
        char buf[len];
        memset(buf, 0x00, len);
        if(copy_from_user(buf, (char *)arg, len-1))
            return -EFAULT;
        down_write(&uts_sem);
        strncpy(cp, buf, len);
        up_write(&uts_sem);
    }
    return 0;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 */
static int dev_open(struct inode *inp, struct file *fp)
{
    MOD_INC_USE_COUNT;                         /* Increment usage count    */
    return 0;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *  This closes the device.
 */
static int dev_close(struct inode *inp, struct file *fp)
{
    MOD_DEC_USE_COUNT;                         /* Decrement usage count */
    return 0;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *   Initialize and register the module
 */
int __init init_module()
{
    int result;
    if((ops = (SFO *) kmalloc(sizeof(SFO), GFP_KERNEL)) == NULL)
    {
        printk(KERN_ERR"%s : Can't allocate memory\n", dev);
        return -ENOMEM;
    }
    memset(ops, 0x00, sizeof(SFO));
    ops->owner   = THIS_MODULE;
    ops->ioctl   = dev_ioctl;
    ops->open    = dev_open;
    ops->release = dev_close;
    if((result = register_chrdev(DEV_MAJOR, dev, ops)) < 0)
    {
        printk(KERN_ERR"%s : Can't register major %d\n", dev, DEV_MAJOR);
        return result;
    }
    printk(KERN_INFO"%s : initialization complete\n", dev);
    return 0;
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*
 *   Release the module.
 */
void cleanup_module()
{
    int result;
    if((result = unregister_chrdev(DEV_MAJOR, dev)) < 0)
        printk(KERN_ALERT"%s : Can't unregister (%d)\n", dev, result);
    else
        printk(KERN_INFO"%s : unregistered. Bye!\n", dev);
    kfree(ops);
}
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
